@@ -372,18 +372,18 @@ GetVectParam.multivariateNormalClass<-function(object)
     nStates <- object$nStates
     nParam <- as.integer(dimObs + dimObs*(dimObs+1)/2)*nStates
     Res <- rep(0, nParam)
-# Moyennes, puis variances puis corrlations
+
     k <- 1
     for (i in 1:nStates)
     {   Res[k:(k+dimObs-1)] <- object$mean[[i]]
         k <- k + dimObs
-        Res[k:(k+dimObs-1)] <- diag(object$cov[[i]])
-        k <- k + dimObs
-        aa<-diag(1/sqrt(diag(object$cov[[i]])))
-        matCor <- aa%*%object$cov[[i]]%*%aa
-        for (n in 1:(dimObs-1))
-        {   for (m in (n+1):dimObs)
-            {   Res[k] <- matCor[m,n]
+#        Res[k:(k+dimObs-1)] <- diag(object$cov[[i]])
+#        k <- k + dimObs
+#        aa<-diag(1/sqrt(diag(object$cov[[i]])))
+#        matCor <- aa%*%object$cov[[i]]%*%aa
+        for (n in 1:dimObs)
+        {   for (m in n:dimObs)
+            {   Res[k] <- object$cov[[i]][m,n]
                 k <- k + 1
             }
         }
@@ -400,22 +400,20 @@ SetVectParam.multivariateNormalClass<-function(object, Vect)
 {
     dimObs <- object$dimObs
     nStates <- object$nStates
-# Moyennes, puis variances puis corrlations
     k <- 1
     distr <- object
     for (i in 1:nStates)
     {   distr$mean[[i]] <- Vect[k:(k+dimObs-1)]
         k <- k + dimObs
-        aa <- sqrt(diag(Vect[k:(k+dimObs-1)]))
-        k <- k + dimObs
-        matCor <- diag(rep(1, dimObs))
-        for (n in 1:(dimObs-1))
-        {   for (m in (n+1):dimObs)
-            {  matCor[m,n] <- matCor[n,m] <- Vect[k]
+#        aa <- sqrt(diag(Vect[k:(k+dimObs-1)]))
+#        k <- k + dimObs
+#        matCor <- diag(rep(1, dimObs))
+        for (n in 1:dimObs)
+        {   for (m in n:dimObs)
+            {  distr$cov[[i]][m,n] <- distr$cov[[i]][n,m] <- Vect[k]
                 k <- k + 1
             }
         }
-        distr$cov[[i]] <- aa %*% matCor %*% aa
     }
     return(distr)
 }
@@ -463,15 +461,19 @@ GetVectParam.mixtureUnivariateNormalClass<-function(object)
     nProba <- nMixt - 1
     nParam <- (nMean + nVar + nProba)*nStates
     Res <- rep(0, nParam)
-#Les NMixt moyennes, puis les NMixt var puis les NMixt-1 proba
+
     k <- 1
     for (i in 1:nStates)
-    {   Res[k:(k+nMixt-1)] <- object$mean[[i]]
-        k <- k + nMixt
-        Res[k:(k+nMixt-1)] <- object$var[[i]]
-        k <- k + nMixt
-        Res[k:(k+nMixt-2)] <- object$prop[[i]][1:(nMixt-1)]
-        k <- k + nMixt - 1
+    {   for (j in 1:nMixt)
+        {   Res[k] <- object$mean[[i]][j]
+            k <- k + 1
+            Res[k] <- object$var[[i]][j]
+            k <- k + 1
+            if (j < nMixt)
+            {   Res[k] <- object$prop[[i]][j]
+                k <- k + 1
+            }
+        }
     }
     return(Res)
  }
@@ -483,15 +485,14 @@ GetVectAllParam.mixtureUnivariateNormalClass<-function(object)
     nMean <- nVar <- nMixt
     nParam <- (nMean + nVar + nMixt)*nStates
     Res <- rep(0, nParam)
-#Les NMixt moyennes, puis les NMixt var puis les NMixt-1 proba
     k <- 1
     for (i in 1:nStates)
-    {   Res[k:(k+nMixt-1)] <- object$mean[[i]]
-        k <- k + nMixt
-        Res[k:(k+nMixt-1)] <- object$var[[i]]
-        k <- k + nMixt
-        Res[k:(k+nMixt-1)] <- object$prop[[i]]
-        k <- k + nMixt
+    {   for (j in 1:nMixt)
+        {   Res[k] <- object$mean[[i]][j]
+            Res[k+1] <- object$var[[i]][j]
+            Res[k+2] <- object$prop[[i]][j]
+            k <- k + 3
+        }
     }
     return(Res)
  }
@@ -503,16 +504,22 @@ SetVectParam.mixtureUnivariateNormalClass<-function(object, Vect)
     nMean <- nVar <- nMixt
     nProba <- nMixt - 1
     distr <- object
-#Les NMixt moyennes, puis les NMixt var puis les NMixt-1 proba
+
     k <- 1
     for (i in 1:nStates)
-    {   distr$mean[[i]] <- Vect[k:(k+nMixt-1)]
-        k <- k + nMixt
-        distr$var[[i]] <- Vect[k:(k+nMixt-1)]
-        k <- k + nMixt
-        distr$prop[[i]][1:(nMixt-1)] <- Vect[k:(k+nMixt-2)]
-        distr$prop[[i]][nMixt] <- 1.0 -sum(distr$prop[[i]][1:(nMixt-1)])
-        k <- k + nMixt - 1
+    {   for (j in 1:nMixt)
+        {   distr$mean[[i]][j] <- Vect[k]
+            k <- k + 1
+            distr$var[[i]][j] <- Vect[k]
+            k <- k + 1
+            if (j < nMixt)
+            {   distr$prop[[i]][j] <- Vect[k]
+                k <- k + 1
+            }
+            else
+            {   distr$prop[[i]][nMixt] <- 1.0 -sum(distr$prop[[i]][1:(nMixt-1)])
+            }
+        }
     }
     return(distr)
  }
@@ -523,15 +530,14 @@ SetVectAllParam.mixtureUnivariateNormalClass<-function(object, Vect)
     nMixt <- object$nMixt
     nMean <- nVar <- nMixt
     distr <- object
-#Les NMixt moyennes, puis les NMixt var puis les NMixt-1 proba
     k <- 1
     for (i in 1:nStates)
-    {   distr$mean[[i]] <- Vect[k:(k+nMixt-1)]
-        k <- k + nMixt
-        distr$var[[i]] <- Vect[k:(k+nMixt-1)]
-        k <- k + nMixt
-        distr$prop[[i]] <- Vect[k:(k+nMixt-1)]
-        k <- k + nMixt
+    {   for (j in 1:nMixt)
+        {   distr$mean[[i]][j] <- Vect[k]
+            distr$var[[i]][j] <- Vect[k+1]
+            distr$prop[[i]][j] <- Vect[k+2]
+            k <- k + 3
+        }
     }
     return(distr)
  }
@@ -557,7 +563,6 @@ GradConstraint.mixtureUnivariateNormalClass<-function(object)
 }
 
 
-
 GetNParam.mixtureMultivariateNormalClass<-function(object)
 {
     dimObs <- object$dimObs
@@ -594,30 +599,23 @@ GetVectParam.mixtureMultivariateNormalClass<-function(object)
     nParam <- (nMean + nVar + nCor + nProba)*nStates
     Res <- rep(0, nParam)
     k <- 1
-# Les Nmixt x dimObs mean, puis les NMixt*dimObs Var, puis les cor, puis les proba
+
     for (i in 1:nStates)
     {   for (j in 1:nMixt)
         {   Res[k:(k+dimObs-1)] <- object$mean[[i]][[j]]
             k <- k + dimObs
-        }
-        for (j in 1:nMixt)
-        {   aa <- diag(object$cov[[i]][[j]])
-            Res[k:(k+dimObs-1)] <- aa
-            k <- k + dimObs
-        }
-        for (j in 1:nMixt)
-        {   aa <- object$cov[[i]][[j]]
-            bb<-diag(1/sqrt(diag(aa)))
-            matCor <- bb%*%aa%*%bb
-            for (n in 1:(dimObs-1))
-            {   for (m in (n+1):dimObs)
-                {   Res[k] <- matCor[m,n]
+            aa <- object$cov[[i]][[j]]
+            for (n in 1:dimObs)
+            {   for (m in n:dimObs)
+                {   Res[k] <- aa[m,n]
                     k <- k + 1
                 }
             }
+            if ( j < nMixt)
+            {   Res[k] <- object$proportion[[i]][j]
+                k <- k + 1
+            }
         }
-        Res[k:(k+nMixt-2)] <- object$proportion[[i]][1:(nMixt-1)]
-        k <- k + nMixt -1
     }
     return(Res)
 }
@@ -632,32 +630,22 @@ GetVectAllParam.mixtureMultivariateNormalClass<-function(object)
    nParam <- (nMean + nVar + nCor + nMixt)*nStates
     Res <- rep(0, nParam)
     k <- 1
-# Les Nmixt x dimObs mean, puis les NMixt*dimObs Var, puis les cor, puis les proba
     for (i in 1:nStates)
     {   for (j in 1:nMixt)
         {   Res[k:(k+dimObs-1)] <- object$mean[[i]][[j]]
             k <- k + dimObs
-        }
-        for (j in 1:nMixt)
-        {   aa <- diag(object$cov[[i]][[j]])
-            Res[k:(k+dimObs-1)] <- aa
-            k <- k + dimObs
-        }
-        for (j in 1:nMixt)
-        {   aa <- object$cov[[i]][[j]]
-            bb<-diag(1/sqrt(diag(aa)))
-            matCor <- bb%*%aa%*%bb
-            for (n in 1:(dimObs-1))
-            {   for (m in (n+1):dimObs)
-                {   Res[k] <- matCor[m,n]
+            aa <- object$cov[[i]][[j]]
+            for (n in 1:dimObs)
+            {   for (m in n:dimObs)
+                {   Res[k] <- aa[m,n]
                     k <- k + 1
                 }
             }
-        }
-        Res[k:(k+nMixt-1)] <- object$proportion[[i]]
-        k <- k + nMixt
+            Res[k] <- object$proportion[[i]][j]
+            k <- k + 1
+         }
     }
-    return(Res)
+   return(Res)
 }
 
 SetVectParam.mixtureMultivariateNormalClass<-function(object, Vect)
@@ -667,31 +655,26 @@ SetVectParam.mixtureMultivariateNormalClass<-function(object, Vect)
     nMixt <- object$nMixt
     k <- 1
     distr <- object
-# Les Nmixt x dimObs mean, puis les NMixt*dimObs Var, puis les cor, puis les proba
+    aa <- matrix(0, nrow=dimObs, ncol=dimObs)
     for (i in 1:nStates)
     {   for (j in 1:nMixt)
         {   distr$mean[[i]][[j]] <- Vect[k:(k+dimObs-1)]
             k <- k + dimObs
-        }
-        for (j in 1:nMixt)
-        {   distr$cov[[i]][[j]] <- diag(Vect[k:(k+dimObs-1)])
-            k <- k + dimObs
-        }
-
-        for (j in 1:nMixt)
-        {   matCor <- diag(rep(1, dimObs))
-            for (n in 1:(dimObs-1))
-            {   for (m in (n+1):dimObs)
-                {   matCor[m,n] <- matCor[n,m] <- Vect[k]
+            for (n in 1:dimObs)
+            {   for (m in n:dimObs)
+                {   aa[m,n] <- aa[n,m] <- Vect[k]
                     k <- k + 1
                 }
             }
-            aa <- sqrt(distr$cov[[i]][[j]])
-            distr$cov[[i]][[j]] <- aa %*% matCor %*% aa
+            distr$cov[[i]][[j]] <- aa
+            if (j < nMixt)
+            {   distr$proportion[[i]][j] <- Vect[k]
+                k <- k + 1
+            }
+            else
+            {   distr$proportion[[i]][nMixt] <- 1.0 - sum(distr$proportion[[i]][1:(nMixt-1)])
+            }
         }
-        distr$proportion[[i]][1:(nMixt-1)] <- Vect[k:(k+nMixt-2)]
-        distr$proportion[[i]][nMixt] <- 1.0 - sum(distr$proportion[[i]][1:(nMixt-1)])
-        k <- k + nMixt -1
     }
     return(distr)
 }
@@ -703,30 +686,19 @@ SetVectAllParam.mixtureMultivariateNormalClass<-function(object, Vect)
     nMixt <- object$nMixt
     k <- 1
     distr <- object
-# Les Nmixt x dimObs mean, puis les NMixt*dimObs Var, puis les cor, puis les proba
     for (i in 1:nStates)
     {   for (j in 1:nMixt)
         {   distr$mean[[i]][[j]] <- Vect[k:(k+dimObs-1)]
             k <- k + dimObs
-        }
-        for (j in 1:nMixt)
-        {   distr$cov[[i]][[j]] <- diag(Vect[k:(k+dimObs-1)])
-            k <- k + dimObs
-        }
-
-        for (j in 1:nMixt)
-        {   matCor <- diag(rep(1, dimObs))
-            for (n in 1:(dimObs-1))
-            {   for (m in (n+1):dimObs)
-                {   matCor[m,n] <- matCor[n,m] <- Vect[k]
+            for (n in 1:dimObs)
+            {   for (m in n:dimObs)
+                {   distr$cov[[i]][[j]][m,n] <- distr$cov[[i]][[j]][n,m] <- Vect[k]
                     k <- k + 1
                 }
             }
-            aa <- sqrt(distr$cov[[i]][[j]])
-            distr$cov[[i]][[j]] <- aa %*% matCor %*% aa
+           distr$proportion[[i]][j] <- Vect[k]
+           k <- k + 1
         }
-        distr$proportion[[i]] <- Vect[k:(k+nMixt-1)]
-        k <- k + nMixt
     }
     return(distr)
 }
@@ -797,10 +769,12 @@ SetVectParam.discreteClass <- function(object, Vect)
     nStates <- object$nStates
     nLevels <- object$nLevels
     distr <- object
+    noms <- names(object$proba[[1]])
     k <- 1
     for (i in 1:nStates)
     {   distr$proba[[i]][1:(nLevels-1)] <-  Vect[k:(k+nLevels-2)]
         distr$proba[[i]][nLevels] <- 1.0 -sum(distr$proba[[i]][1:(nLevels-1)])
+        names(distr$proba[[i]]) <- noms
         k <- k+nLevels-1
     }
     return(distr)
@@ -811,9 +785,11 @@ SetVectAllParam.discreteClass <- function(object, Vect)
     nStates <- object$nStates
     nLevels <- object$nLevels
     distr <- object
+    noms <- names(object$proba[[1]])
     k <- 1
     for (i in 1:nStates)
     {   distr$proba[[i]] <-  Vect[k:(k+nLevels-1)]
+        names(distr$proba[[i]]) <- noms
         k <- k+nLevels
     }
     return(distr)
@@ -950,70 +926,120 @@ ComputeHessian1 <- function(HMM, obs, asymptMethod)
         return(Hess)
 }
 
-
-asymptoticCovMat <- function(HMM, obs, asymptMethod=c('nlme', 'optim'))
+computeScoreAndInformation <- function(HMM, obs)
 {
     if ( ( class(HMM) != "HMMFitClass" ) && (class(HMM) != "HMMClass") )
         stop("class(HMM) must be 'HMMClass' or 'HMMFitClass'\n")
     if (class(HMM) == "HMMFitClass")
         HMM <- HMM$HMM
-    Hh <- ComputeHessian(HMM, obs, asymptMethod[1])
-    nParam <- dim(Hh)[1]
-    K <- GradConstraint(HMM)
-    Dh <- Hh + t(K) %*% K
-    Dm1 <- try(solve(Dh), silent=TRUE)
-    if (class(Dm1)=="try-error")
-    {   asymptMatCov <- matrix(NaN, ncol=nParam, nrow=nParam)
-        colnames(asymptMatCov) <- rownames(asymptMatCov) <- NomsParamHMM(HMM)
-        warning("Hessian matrix is not inversible")
-        return(asymptMatCov)
-    }
-    Aux <- K %*% Dm1 %*% t(K)
-    Auxm1 <- try(solve(Aux), silent=TRUE)
-    if (class(Auxm1)=="try-error")
-    {   asymptMatCov <- matrix(NaN, ncol=nParam, nrow=nParam)
-        colnames(asymptMatCov) <- rownames(asymptMatCov) <- NomsParamHMM(HMM)
-        warning("Hessian matrix is not inversible")
-        return(asymptMatCov)
-    }
-    asymptMatCov <- Dm1 - Dm1 %*% t(K) %*% Auxm1 %*% K %*% Dm1
-    if (is.list(obs))
-    {   nTot <- length(obs[[1]])
-        nSample <- length(obs)
-        if (nSample > 1)
-        {   for (j in 2:nSample)
-                nTot <- nTot + length(obs[[j]])
-        }
-    }
-    else
-        nTot <- length(obs)
 
-    colnames(asymptMatCov) <- rownames(asymptMatCov) <- NomsParamHMM(HMM)
-    return(asymptMatCov)
+    if (is.data.frame(obs))
+    {   dimObs <- dim(obs)[2]
+        if (dimObs == 1)
+            obs <- obs[,1]
+        else
+            obs <- as.matrix(obs[,1:dimObs])
+    }
+
+    maListe <- TransfListe(HMM$distribution, obs)
+    HMM <- setStorageMode(HMM)
+
+    Res <- .Call("RScoreAndInformation", HMM, maListe$Zt)
+    score <- Res[[1]]
+    information <- Res[[2]]
+    noms <- NomsIndepParamHMM(HMM)
+    names(score) <- noms
+    row.names(information) <- noms
+    colnames(information) <- noms
+    return(list(score=score, information=information))
 }
 
-asymptoticSimCovMat <- function(HMM, obs, nSimul, verbose=FALSE, oldCovMat=NULL, oldNSimul=0)
+asymptoticCov <- function(HMM, obs)
 {
     if ( ( class(HMM) != "HMMFitClass" ) && (class(HMM) != "HMMClass") )
         stop("class(HMM) must be 'HMMClass' or 'HMMFitClass'\n")
     if (class(HMM) == "HMMFitClass")
         HMM <- HMM$HMM
 
-    if (is.null(oldCovMat))
-    {
-        Res <- asymptoticIterSimCovMat(HMM, obs, nSimul, verbose)
+    if (is.data.frame(obs))
+    {   dimObs <- dim(obs)[2]
+        if (dimObs == 1)
+            obs <- obs[,1]
+        else
+            obs <- as.matrix(obs[,1:dimObs])
     }
-    else
-    {   Teta <- GetVectAllParam(HMM)
-        esp2Mat <- Teta %*% t(Teta)
-        espVect <- Teta
-        nSimul <- oldNSimul
-        old <- list(cov=oldCovMat, esp2Mat=esp2Mat, mean = espVect, nSimul=oldNSimul)
-        Res <- asymptoticIterSimCovMat(HMM, obs, nSimul, verbose, old)
-    }
-    warning("asymtoticSimCovMat is deprecated. Use asymptoticIterSimCovMat")
-    return(Res$cov)
- }
+
+    maListe <- TransfListe(HMM$distribution, obs)
+    HMM <- setStorageMode(HMM)
+
+    Res <- .Call("RComputeCov", HMM, maListe$Zt)
+    colnames(Res) <- rownames(Res) <- NomsParamHMM(HMM)
+    return(Res)
+}
+
+#asymptoticCovMat <- function(HMM, obs, asymptMethod=c('nlme', 'optim'))
+#{
+#    if ( ( class(HMM) != "HMMFitClass" ) && (class(HMM) != "HMMClass") )
+#        stop("class(HMM) must be 'HMMClass' or 'HMMFitClass'\n")
+#    if (class(HMM) == "HMMFitClass")
+#        HMM <- HMM$HMM
+#   Hh <- ComputeHessian(HMM, obs, asymptMethod[1])
+#    nParam <- dim(Hh)[1]
+#    K <- GradConstraint(HMM)
+#    Dh <- Hh + t(K) %*% K
+#    Dm1 <- try(solve(Dh), silent=TRUE)
+#    if (class(Dm1)=="try-error")
+#    {   asymptMatCov <- matrix(NaN, ncol=nParam, nrow=nParam)
+#        colnames(asymptMatCov) <- rownames(asymptMatCov) <- NomsParamHMM(HMM)
+#        warning("Hessian matrix is not inversible")
+#        return(asymptMatCov)
+#    }
+#    Aux <- K %*% Dm1 %*% t(K)
+#    Auxm1 <- try(solve(Aux), silent=TRUE)
+#    if (class(Auxm1)=="try-error")
+#    {   asymptMatCov <- matrix(NaN, ncol=nParam, nrow=nParam)
+#        colnames(asymptMatCov) <- rownames(asymptMatCov) <- NomsParamHMM(HMM)
+#        warning("Hessian matrix is not inversible")
+#        return(asymptMatCov)
+#    }
+#    asymptMatCov <- Dm1 - Dm1 %*% t(K) %*% Auxm1 %*% K %*% Dm1
+#    if (is.list(obs))
+#    {   nTot <- length(obs[[1]])
+#        nSample <- length(obs)
+#        if (nSample > 1)
+#        {   for (j in 2:nSample)
+#                nTot <- nTot + length(obs[[j]])
+#        }
+#    }
+#    else
+#        nTot <- length(obs)
+#
+#    colnames(asymptMatCov) <- rownames(asymptMatCov) <- NomsParamHMM(HMM)
+#    return(asymptMatCov)
+#}
+
+#asymptoticSimCovMat <- function(HMM, obs, nSimul, verbose=FALSE, oldCovMat=NULL, oldNSimul=0)
+#{
+#    if ( ( class(HMM) != "HMMFitClass" ) && (class(HMM) != "HMMClass") )
+#        stop("class(HMM) must be 'HMMClass' or 'HMMFitClass'\n")
+#    if (class(HMM) == "HMMFitClass")
+#        HMM <- HMM$HMM
+#
+#    if (is.null(oldCovMat))
+#    {
+#        Res <- asymptoticIterSimCovMat(HMM, obs, nSimul, verbose)
+#    }
+#    else
+#    {   Teta <- GetVectAllParam(HMM)
+#        esp2Mat <- Teta %*% t(Teta)
+#        espVect <- Teta
+#        nSimul <- oldNSimul
+#        old <- list(cov=oldCovMat, esp2Mat=esp2Mat, mean = espVect, nSimul=oldNSimul)
+#        Res <- asymptoticIterSimCovMat(HMM, obs, nSimul, verbose, old)
+#    }
+#    warning("asymtoticSimCovMat is deprecated. Use asymptoticIterSimCovMat")
+#    return(Res$cov)
+# }
 
 asymptoticIterSimCovMat <- function(HMM, obs, nSimul, verbose=FALSE, oldCovMat=NULL)
 {
@@ -1072,49 +1098,46 @@ asymptoticIterSimCovMat <- function(HMM, obs, nSimul, verbose=FALSE, oldCovMat=N
 }
 
 
-asymptoticCovMat <- function(HMM, obs, asymptMethod=c('nlme', 'optim'))
-{
-    if ( ( class(HMM) != "HMMFitClass" ) && (class(HMM) != "HMMClass") )
-        stop("class(HMM) must be 'HMMClass' or 'HMMFitClass'\n")
-    if (class(HMM) == "HMMFitClass")
-        HMM <- HMM$HMM
-    Hh <- ComputeHessian(HMM, obs, asymptMethod[1])
-    nParam <- dim(Hh)[1]
-    K <- GradConstraint(HMM)
-    Dh <- Hh + t(K) %*% K
-    Dm1 <- try(solve(Dh), silent=TRUE)
-    if (class(Dm1)=="try-error")
-    {   asymptMatCov <- matrix(NaN, ncol=nParam, nrow=nParam)
-        colnames(asymptMatCov) <- rownames(asymptMatCov) <- NomsParamHMM(HMM)
-        warning("Hessian matrix is not inversible")
-        return(asymptMatCov)
-    }
-    Aux <- K %*% Dm1 %*% t(K)
-    Auxm1 <- try(solve(Aux), silent=TRUE)
-    if (class(Auxm1)=="try-error")
-    {   asymptMatCov <- matrix(NaN, ncol=nParam, nrow=nParam)
-        colnames(asymptMatCov) <- rownames(asymptMatCov) <- NomsParamHMM(HMM)
-        warning("Hessian matrix is not inversible")
-        return(asymptMatCov)
-    }
-    asymptMatCov <- Dm1 - Dm1 %*% t(K) %*% Auxm1 %*% K %*% Dm1
-    if (is.list(obs))
-    {   nTot <- length(obs[[1]])
-        nSample <- length(obs)
-        if (nSample > 1)
-        {   for (j in 2:nSample)
-                nTot <- nTot + length(obs[[j]])
-        }
-    }
-    else
-        nTot <- length(obs)
-
-    colnames(asymptMatCov) <- rownames(asymptMatCov) <- NomsParamHMM(HMM)
-    return(asymptMatCov)
-}
-
-
-
+#asymptoticCovMat <- function(HMM, obs, asymptMethod=c('nlme', 'optim'))
+#{
+#    if ( ( class(HMM) != "HMMFitClass" ) && (class(HMM) != "HMMClass") )
+#        stop("class(HMM) must be 'HMMClass' or 'HMMFitClass'\n")
+#    if (class(HMM) == "HMMFitClass")
+#        HMM <- HMM$HMM
+#    Hh <- ComputeHessian(HMM, obs, asymptMethod[1])
+#    nParam <- dim(Hh)[1]
+#    K <- GradConstraint(HMM)
+#    Dh <- Hh + t(K) %*% K
+#    Dm1 <- try(solve(Dh), silent=TRUE)
+#    if (class(Dm1)=="try-error")
+#    {   asymptMatCov <- matrix(NaN, ncol=nParam, nrow=nParam)
+#        colnames(asymptMatCov) <- rownames(asymptMatCov) <- NomsParamHMM(HMM)
+#        warning("Hessian matrix is not inversible")
+#        return(asymptMatCov)
+#    }
+#    Aux <- K %*% Dm1 %*% t(K)
+#    Auxm1 <- try(solve(Aux), silent=TRUE)
+#    if (class(Auxm1)=="try-error")
+#    {   asymptMatCov <- matrix(NaN, ncol=nParam, nrow=nParam)
+#        colnames(asymptMatCov) <- rownames(asymptMatCov) <- NomsParamHMM(HMM)
+#        warning("Hessian matrix is not inversible")
+#        return(asymptMatCov)
+#    }
+#    asymptMatCov <- Dm1 - Dm1 %*% t(K) %*% Auxm1 %*% K %*% Dm1
+#    if (is.list(obs))
+#    {   nTot <- length(obs[[1]])
+#        nSample <- length(obs)
+#        if (nSample > 1)
+#        {   for (j in 2:nSample)
+#                nTot <- nTot + length(obs[[j]])
+#        }
+#    }
+#    else
+#        nTot <- length(obs)
+#
+#    colnames(asymptMatCov) <- rownames(asymptMatCov) <- NomsParamHMM(HMM)
+#    return(asymptMatCov)
+#}
 
 setAsymptoticCovMat<-function(HMMFit, asymptCovMat)
 {
@@ -1131,16 +1154,26 @@ setAsymptoticCovMat<-function(HMMFit, asymptCovMat)
 
 
 NomsParamHMM <- function(object) UseMethod("NomsParamHMM")
+NomsIndepParamHMM <- function(object) UseMethod("NomsIndepParamHMM")
 
 NomsParamHMM.default <- function(object)
 {   return(NULL)
 }
 
+NomsIndepParamHMM.default <- function(object)
+{   return(NULL)
+}
+
 NomsParamHMM.distributionClass <- function(object)
 {
     return(NextMethod(generic="NomsParamHMM", object=object))
 }
 
+NomsIndepParamHMM.distributionClass <- function(object)
+{
+    return(NextMethod(generic="NomsIndepParamHMM", object=object))
+}
+
 NomsParamHMM.discreteClass <- function(object)
 {
     Res <- NULL
@@ -1162,6 +1195,27 @@ NomsParamHMM.discreteClass <- function(object)
     return(Res)
 }
 
+NomsIndepParamHMM.discreteClass <- function(object)
+{
+    Res <- NULL
+    nStates <- object$nStates
+    nLevels <- object$nLevels
+    namesProba <- names(object$proba[[1]])
+    if (is.null(namesProba))
+    {   for (j in 1:(nLevels-1))
+        {   Aux <- sprintf("p[%d]", j)
+            namesProba <- c(Res, Aux)
+        }
+    }
+    for (i in 1:nStates)
+    {   for (j in 1:(nLevels-1))
+        {   Aux <- sprintf("State[%d]-%s", i, namesProba[j])
+            Res <- c(Res, Aux)
+        }
+    }
+    return(Res)
+}
+
 NomsParamHMM.univariateNormalClass <- function(object)
 {
     Res <- NULL
@@ -1176,6 +1230,11 @@ NomsParamHMM.univariateNormalClass <- function(object)
 
 }
 
+NomsIndepParamHMM.univariateNormalClass <- function(object)
+{
+    return(NomsParamHMM(object))
+}
+
 NomsParamHMM.mixtureUnivariateNormalClass <- function(object)
 {
     Res <- NULL
@@ -1185,16 +1244,31 @@ NomsParamHMM.mixtureUnivariateNormalClass <- function(object)
     {   for (j in 1:nMixt)
         {   Aux <- sprintf("state[%d]-mean[%d]", i, j)
             Res <- c(Res, Aux)
-        }
-        for (j in 1:nMixt)
-        {   Aux <- sprintf("state[%d]-var[%d]", i, j)
+            Aux <- sprintf("state[%d]-var[%d]", i, j)
             Res <- c(Res, Aux)
-        }
-        for (j in 1:nMixt)
-        {   Aux <- sprintf("state[%d]-prop[%d]", i, j)
+            Aux <- sprintf("state[%d]-prop[%d]", i, j)
             Res <- c(Res, Aux)
         }
+    }
+    return(Res)
+}
 
+NomsIndepParamHMM.mixtureUnivariateNormalClass <- function(object)
+{
+    Res <- NULL
+    nStates <- object$nStates
+    nMixt <- object$nMixt
+    for (i in 1:nStates)
+    {   for (j in 1:nMixt)
+        {   Aux <- sprintf("state[%d]-mean[%d]", i, j)
+            Res <- c(Res, Aux)
+            Aux <- sprintf("state[%d]-var[%d]", i, j)
+            Res <- c(Res, Aux)
+            if (j < nMixt)
+            {   Aux <- sprintf("state[%d]-prop[%d]", i, j)
+                Res <- c(Res, Aux)
+            }
+        }
     }
     return(Res)
 }
@@ -1210,13 +1284,13 @@ NomsParamHMM.multivariateNormalClass <- function(object)
             Res <- c(Res, Aux)
         }
         for (j in 1:dimObs)
-        {   Aux <- sprintf("state[%d]-var[%d]", i, j)
-            Res <- c(Res, Aux)
-        }
-
-        for (j in 1:(dimObs-1))
-        {   for (k in (j+1):dimObs)
-            {   Aux <- sprintf("state[%d]-Cor[%d,%d]", i, j, k)
+        {   for (k in j:dimObs)
+            {   if (j == k)
+                {    Aux <- sprintf("state[%d]-var[%d,%d]", i, j, j)
+                }
+                else
+                {   Aux <- sprintf("state[%d]-cov[%d,%d]", i, j, k)
+                }
                 Res <- c(Res, Aux)
             }
         }
@@ -1224,6 +1298,11 @@ NomsParamHMM.multivariateNormalClass <- function(object)
     return(Res)
 }
 
+NomsIndepParamHMM.multivariateNormalClass <- function(object)
+{
+    return(NomsParamHMM(object))
+}
+
 NomsParamHMM.mixtureMultivariateNormalClass <- function(object)
 {
     Res <- NULL
@@ -1237,21 +1316,51 @@ NomsParamHMM.mixtureMultivariateNormalClass <- function(object)
                 Res <- c(Res, Aux)
             }
             for (j in 1:dimObs)
-            {   Aux <- sprintf("state[%d]-Mixt[%d]-var[%d]", i, m, j)
-                Res <- c(Res, Aux)
-            }
-            for (j in 1:(dimObs-1))
-            {   for (k in (j+1):dimObs)
-                {   Aux <- sprintf("state[%d]-Mixt[%d]-Cor[%d,%d]", i, m, j, k)
+            {   for (k in j:dimObs)
+                {   if (j == k)
+                    {   Aux <- sprintf("state[%d]-Mixt[%d]-var[%d,%d]", i, m, j, j)
+                    }
+                    else
+                    {   Aux <- sprintf("state[%d]-Mixt[%d]-Cov[%d,%d]", i, m, j, k)
+                    }
                     Res <- c(Res, Aux)
                 }
             }
-        }
-        for (j in 1:nMixt)
-        {   Aux <- sprintf("state[%d]-prop[%d]", i, j)
+            Aux <- sprintf("state[%d]-prop[%d]", i, m)
             Res <- c(Res, Aux)
         }
+    }
+    return(Res)
+}
 
+NomsIndepParamHMM.mixtureMultivariateNormalClass <- function(object)
+{
+    Res <- NULL
+    nStates <- object$nStates
+    dimObs <- object$dimObs
+    nMixt <- object$nMixt
+    for (i in 1:nStates)
+    {   for (m in 1:nMixt)
+        {   for (j in 1:dimObs)
+            {   Aux <- sprintf("state[%d]-Mixt[%d]-mean[%d]", i, m, j)
+                Res <- c(Res, Aux)
+            }
+            for (j in 1:dimObs)
+            {   for (k in j:dimObs)
+                {   if (j == k)
+                    {   Aux <- sprintf("state[%d]-Mixt[%d]-var[%d,%d]", i, m, j, j)
+                    }
+                    else
+                    {   Aux <- sprintf("state[%d]-Mixt[%d]-Cov[%d,%d]", i, m, j, k)
+                    }
+                    Res <- c(Res, Aux)
+                }
+            }
+            if (m < nMixt)
+            {   Aux <- sprintf("state[%d]-prop[%d]", i, m)
+                Res <- c(Res, Aux)
+            }
+        }
     }
     return(Res)
 }
@@ -1275,11 +1384,35 @@ NomsParamHMM.HMMClass <- function(object)
     return(Noms)
 }
 
+NomsIndepParamHMM.HMMClass <- function(object)
+{
+    nStates <- object$distribution$nStates
+    Noms <- NULL
+    for (i in 1:(nStates-1))
+    {   Aux <- sprintf("Pi[%d]", i)
+        Noms <- c(Noms, Aux)
+    }
+
+    for (i in 1:nStates)
+        for (j in 1:(nStates-1))
+        {   Aux <- sprintf("transMat[%d,%d]", i, j)
+            Noms <- c(Noms, Aux)
+        }
+    Aux <- NomsIndepParamHMM(object$distribution)
+    Noms <- c(Noms, Aux)
+    return(Noms)
+}
+
 NomsParamHMM.HMMFitClass <- function(object)
 {
     return(NomsParamHMM(object$HMM))
 }
 
+NomsIndepParamHMM.HMMFitClass <- function(object)
+{
+    return(NomsIndepParamHMM(object$HMM))
+}
+
 summary.HMMFitClass <-function (object, ...)
 {
     ans = NULL
@@ -1308,7 +1441,7 @@ summary.HMMFitClass <-function (object, ...)
     ans$AIC <- object$AIC
     if (is.null(object$asymptCov))
     {   cat(sprintf("Computing the asymptotic covariance matrix of estimates\n"))
-        object$asymptCov <- asymptoticCovMat(object, object$obs)
+        object$asymptCov <- asymptoticCov(object, object$obs)
     }
 
     nAllParam <- GetNAllParam(object)
@@ -1459,9 +1592,6 @@ HMMGetRank.discreteClass<-function(object)
     return(Res)
 }
 
-
-
-
 HMMSort <- function(object, r=NULL) UseMethod("HMMSort")
 HMMSort.HMMFitClass <- function(object, r=NULL)
 {
@@ -1561,12 +1691,12 @@ HMMSort.mixtureMultivariateNormalClass <- function(object, r)
 HMMSort.discreteClass <- function(object, r)
 {
     proba <- object$proba
+    name <- names(proba[[1]])
     proba[r] <- proba
-    label <- names(proba[[1]])
-    if ( label[1] == 'p 1')
-    {   label = NULL
+    if ( name[1] == 'p 1')
+    {   name = NULL
     }
-    return(distributionSet(dis="DISCRETE", proba=proba, labels=label, verif=FALSE))
+    return(distributionSet(dis="DISCRETE", proba=proba, labels=name, verif=FALSE))
 }
 
 
