@@ -27,6 +27,7 @@ double* myDens = new double[myT] ;
 
         delete[] myDens ;
 }
+
 cMixtMultivariateNormal::cMixtMultivariateNormal(uint theNClass, uint theNMixt, uint theDimObs)
 {       MESS_CREAT("cMixtMultivariateNormal")
         mvNClass = theNClass ;
@@ -94,6 +95,101 @@ cDVector myDet = cDVector(mvNMixt) ;
         delete [] myInvCov ;
 }
 
+void cMixtMultivariateNormal::ComputeDerivative(cDVector& theY, cDVector** theGrad, cDMatrix** theHess)
+{
+uint myT = theY.GetSize()/mvDimObs ;
+uint myNCovParam = mvDimObs * (mvDimObs + 1)/2 ;
+uint myNNormParam = mvDimObs + myNCovParam ;
+uint myNParam = mvNMixt*myNNormParam + mvNMixt - 1 ;
+cDVector* myGradNorm = new cDVector[myT] ;
+cDMatrix* myHessNorm = new cDMatrix[myT] ;
+cDVector myDens(myT), myLastDens(myT);
+
+	for (register uint t = 0 ; t < myT ; t++)
+	{	myGradNorm[t].ReAlloc(myNNormParam) ;
+		myHessNorm[t].ReAlloc(myNNormParam, myNNormParam) ;
+	}
+
+	for (register uint j = 0 ; j < mvNClass ; j++)
+	{
+		for (register uint t = 0 ; t < myT ; t++)
+		{	theGrad[j][t] = 0.0 ;
+			theHess[j][t] = 0.0 ;
+		}
+
+	cDMatrix myInvCov(mvDimObs, mvDimObs) ;
+	double myDeterminant ;
+	uint k = (mvNClass - 1)*(mvNClass + 1) + j*myNParam ;
+		
+		LapackInvAndDet(mCov[j][mvNMixt-1], myInvCov, myDeterminant) ;
+		MultivariateNormalDensity(theY, mMean[j][mvNMixt-1], myInvCov, myDeterminant, myLastDens) ;	
+		for (register uint l = 0 ; l < mvNMixt ; l++)
+		{	LapackInvAndDet(mCov[j][l], myInvCov, myDeterminant) ;
+			MultivariateNormalDensity(theY, mMean[j][l], myInvCov, myDeterminant, myDens) ;
+			MultivariateNormalDensityDeriv(theY, mMean[j][l], mCov[j][l], myInvCov, myDeterminant, myDens, myGradNorm, myHessNorm) ;
+			for (register uint t = 0 ; t < myT ; t++)
+			{	SetSubVector(mp[j][l]*myGradNorm[t], k, theGrad[j][t]) ;
+			cDMatrix myAuxMat = mp[j][l]*myHessNorm[t] ;	
+				SetSubMatrix(myAuxMat, k, k, theHess[j][t]) ;
+			
+				if (l < mvNMixt - 1)
+				{	theGrad[j][t][k+myNNormParam] = myDens[t] - myLastDens[t] ;
+					for (register uint p = 0 ; p < myNNormParam ; p++)
+						theHess[j][t][k+myNNormParam][p+k] = theHess[j][t][p+k][k+myNNormParam] = myGradNorm[t][p] ;
+				}					
+			}
+			k += myNNormParam ;
+			if (l < mvNMixt - 1)
+				k++ ;
+		}
+	}
+
+	for (register uint t = 0 ; t < myT ; t++)
+	{	myGradNorm[t].Delete() ;
+		myHessNorm[t].Delete() ;
+	}
+	delete [] myGradNorm ;
+	delete [] myHessNorm ;
+
+}
+
+void cMixtMultivariateNormal::ComputeCov(cDMatrix& theCov)
+{
+uint myBegIndex = (mvNClass - 1) * (mvNClass + 1) ;
+uint myNNormParam = mvDimObs + mvDimObs*(mvDimObs + 1)/2 ;
+uint myNFreeMixt = (myNNormParam + 1)*mvNMixt - 1 ;
+uint mySizeCour = theCov.GetNCols() ;
+cDVector myU(mySizeCour, 0.0) ;
+
+	for (register uint n = 0 ; n < mvNClass ; n++)
+	{
+		for (register uint i = myBegIndex + myNNormParam ; i < myBegIndex + myNFreeMixt ; i+=myNNormParam+1)
+			myU[i] = -1.0 ;
+		theCov = AddOneVariable(theCov, myU) ;
+		mySizeCour++ ;
+		myU.ReAlloc(mySizeCour, 0.0) ;
+		myBegIndex += myNFreeMixt ;
+	}
+
+}
+
+cDVector cMixtMultivariateNormal::GetDistrNumParam(const cDVector& theNumDistrParam, uint& theNextInd)
+{
+uint myNNormParam = mvDimObs + mvDimObs*(mvDimObs + 1)/2 ;
+uint myNFreeParam =(myNNormParam + 1)*mvNMixt - 1 ;
+cDVector myNumDistrParam ;
+cDVector myNumMixt(myNFreeParam) ;
+uint myIndCour = 0 ;
+	for (register uint j = 0 ; j < mvNClass ; j++)
+	{	GetSubVector(theNumDistrParam, myIndCour, myNFreeParam, myNumMixt) ;
+		myNumDistrParam = cat(myNumDistrParam, myNumMixt) ;
+		myNumDistrParam = cat(myNumDistrParam, (double)theNextInd) ;
+		theNextInd++ ;
+		myIndCour += myNFreeParam ;
+	}
+	return myNumDistrParam ;
+}
+
 void cMixtMultivariateNormal::UpdateParameters(cInParam& theInParam, cBaumWelch& theBaumWelch, cDMatrix* theCondProba)
 {       
 cDMatrix*      myInvCov = new cDMatrix[mvNMixt];
@@ -152,7 +248,6 @@ double*         myDet = new double[mvNMixt] ;
         }
 }               
 
-
 void cMixtMultivariateNormal::InitParameters(cBaumWelchInParam &theInParam)
 {
 #ifdef _RDLL_
@@ -212,18 +307,26 @@ double mys = 0.0 ;
 
 void cMixtMultivariateNormal::CopyDistr(cDistribution* theSrc)
 {
-cMixtMultivariateNormal *mySrc ;
-        mySrc = static_cast<cMixtMultivariateNormal *>(theSrc) ;
-        mvNClass = mySrc->mvNClass ;
-        mvDimObs = mySrc->mvDimObs ;
-        mvNMixt = mySrc->mvNMixt ;
-        for (register uint i = 0 ; i < mvNClass ; i++)
-        {       for (register uint l = 0 ; l < mvNMixt ; l++)
-                {       mMean[i][l] = mySrc->mMean[i][l] ;
-                        mCov[i][l] = mySrc->mCov[i][l] ;
-                }
-                mp[i] = mySrc->mp[i] ;
-        }
+cMixtMultivariateNormal* mySrc = dynamic_cast<cMixtMultivariateNormal *>(theSrc) ;
+	if (mySrc)
+	{	mvNClass = mySrc->mvNClass ;
+		mvDimObs = mySrc->mvDimObs ;
+		mvNMixt = mySrc->mvNMixt ;
+		for (register uint i = 0 ; i < mvNClass ; i++)
+		{	for (register uint l = 0 ; l < mvNMixt ; l++)
+			{	mMean[i][l] = mySrc->mMean[i][l] ;
+				mCov[i][l] = mySrc->mCov[i][l] ;
+			}
+			mp[i] = mySrc->mp[i] ;
+		}
+	}
+	else
+		cOTError("Wrong distribution in cMixtMultivariateNormal") ;
+}
+
+cMixtMultivariateNormal::cMixtMultivariateNormal(cDistribution& theSrc)
+{
+	CopyDistr(&theSrc) ;
 }
 
 void cMixtMultivariateNormal::Print()
@@ -244,7 +347,6 @@ void cMixtMultivariateNormal::Print()
         }
 }
 
-
 void cMixtMultivariateNormal::GetParam(uint theDeb, cDVector& theParam)
 {
 register uint k = theDeb ;
@@ -260,6 +362,7 @@ register uint k = theDeb ;
                 }
         }
 }
+
 void cMixtMultivariateNormal::SetParam(uint theDeb, cDVector& theParam)
 {
 register uint k = theDeb ;
