@@ -19,10 +19,14 @@ class cUnivariateNormal : public cDistribution
                 cDVector       mVar    ;
         public :
                 cUnivariateNormal(uint theNClass = 0) ;
+				cUnivariateNormal(cDistribution& theSrc) ;
                 virtual ~cUnivariateNormal() ;
                 void ComputeCondProba(cDVector* theY, uint theNSample, cDMatrix* theCondProba) ;
                 void UpdateParameters(cInParam& theInParam, cBaumWelch& theBaumWelch, cDMatrix* theCondProba=NULL) ;
-                void InitParameters(cBaumWelchInParam &theInParam) ;
+                void ComputeDerivative(cDVector& theY, cDVector** theGrad, cDMatrix** theHess) ;
+				void ComputeCov(cDMatrix& theCov) ;
+				cDVector GetDistrNumParam(const cDVector& theNumDistrParam, uint& theNextInd) ;
+				void InitParameters(cBaumWelchInParam &theInParam) ;
                 void Print() ;
                 void GetParam(uint theDeb, cDVector& theParam) ;
                 void SetParam(uint theDeb, cDVector& theParam) ;
